//--------------------------------------------------------------
// File     : stm32_ub_fpga.c
// Datum    : 21.06.2017
// Version  : 1.0
// Autor    : UB
// EMail    : becker_u(@)gmx.de
// Web      : www.mikrocontroller.bplaced.net
// CPU      : STM32F407
// IDE      : OpenSTM32
// GCC      : 4.9 2015q2
// Module   : CubeHAL
//--------------------------------------------------------------


#include "stm32_ub_fpga.h"


//--------------------------------------------------------------
// internal functions
//--------------------------------------------------------------
void init_gpio(void);
void progb_lo(void);
void progb_hi(void);
void cclk_lo(void);
void cclk_hi(void);
void data_lo(void);
void data_hi(void);
uint8_t read_initb(void);
uint8_t read_done(void);
void prg_byte(uint8_t value);
void send_startup_clk(void);
void fpga_delay(uint32_t delay);


//--------------------------------------------------------------
// fpga init
//--------------------------------------------------------------
void UB_Fpga_Init(void)
{
	init_gpio();
}


//--------------------------------------------------------------
// programm fpga with bitfile in slave serial mode
//--------------------------------------------------------------
uint8_t UB_Fpga_Programm(UB_Bitfile *filename)
{
	uint8_t prg_status = FPGA_PRG_OK;
	uint32_t n;

	// init sequence
	progb_lo();
	fpga_delay(FPGA_DELAY_START);
	if(read_initb() != GPIO_PIN_RESET) prg_status = FPGA_PRG_ERROR1;
	progb_hi();
	fpga_delay(FPGA_DELAY_START);
	if(read_initb() == GPIO_PIN_RESET) prg_status = FPGA_PRG_ERROR2;
	if(read_done() != GPIO_PIN_RESET) prg_status = FPGA_PRG_ERROR3;

	// download bitfile
	for(n=0;n<filename->size;n++) {
		prg_byte(filename->table[n]);
		if(read_initb() == GPIO_PIN_RESET) {
			prg_status = FPGA_PRG_ERROR4;
			break;
		}
	}

	// startup sequence
	data_lo();
	if(prg_status==FPGA_PRG_OK) {
		send_startup_clk();
		if(read_initb() == GPIO_PIN_RESET) prg_status = FPGA_PRG_ERROR5;
		if(read_done() == GPIO_PIN_RESET) prg_status = FPGA_PRG_ERROR6;
	}

	return prg_status;
}


//--------------------------------------------------------------
void prg_byte(uint8_t value)
{
	uint8_t n,data;
	uint8_t mask = 0x80;

	// send single byte
	for(n=0;n<8;n++)
	{
		data = (value & mask);
		if(data != 0) {
			data_hi();
		}
		else {
			data_lo();
		}
		cclk_hi();
		cclk_lo();
		mask = (mask >> 1);
	}
}
//--------------------------------------------------------------
void send_startup_clk(void)
{
	uint8_t n;

	// send n clock cycles
	for(n=0;n<16;n++) {
		if(read_done() != GPIO_PIN_RESET) return;
		cclk_hi();
		cclk_lo();
	}
}
//--------------------------------------------------------------
void init_gpio(void)
{
	GPIO_InitTypeDef GPIO_InitStruct;

	UB_System_ClockEnable(FPGA_PROG_B_PORT);
	UB_System_ClockEnable(FPGA_INIT_B_PORT);
	UB_System_ClockEnable(FPGA_CCLK_PORT);
	UB_System_ClockEnable(FPGA_DATA_PORT);
	UB_System_ClockEnable(FPGA_DONE_PORT);

    // config PROG_B
    GPIO_InitStruct.Pin = FPGA_PROG_B_PIN;
    GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    GPIO_InitStruct.Speed = GPIO_SPEED_LOW;
    HAL_GPIO_Init(FPGA_PROG_B_PORT, &GPIO_InitStruct);
    progb_hi();

    // config INIT_B
    GPIO_InitStruct.Pin = FPGA_INIT_B_PIN;
    GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
    GPIO_InitStruct.Pull = GPIO_PULLUP;
    GPIO_InitStruct.Speed = GPIO_SPEED_LOW;
    HAL_GPIO_Init(FPGA_INIT_B_PORT, &GPIO_InitStruct);

    // config CCLK
    GPIO_InitStruct.Pin = FPGA_CCLK_PIN;
    GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    GPIO_InitStruct.Speed = GPIO_SPEED_LOW;
    HAL_GPIO_Init(FPGA_CCLK_PORT, &GPIO_InitStruct);
    cclk_lo();

    // config DATA
    GPIO_InitStruct.Pin = FPGA_DATA_PIN;
    GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    GPIO_InitStruct.Speed = GPIO_SPEED_LOW;
    HAL_GPIO_Init(FPGA_DATA_PORT, &GPIO_InitStruct);
    data_lo();

    // config DONE
    GPIO_InitStruct.Pin = FPGA_DONE_PIN;
    GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
    GPIO_InitStruct.Pull = GPIO_PULLUP;
    GPIO_InitStruct.Speed = GPIO_SPEED_LOW;
    HAL_GPIO_Init(FPGA_DONE_PORT, &GPIO_InitStruct);
}
//--------------------------------------------------------------
void progb_lo(void)
{
	FPGA_PROG_B_PORT->BSRR = (uint32_t)FPGA_PROG_B_PIN << 16;
}
//--------------------------------------------------------------
void progb_hi(void)
{
	FPGA_PROG_B_PORT->BSRR = FPGA_PROG_B_PIN;
}
//--------------------------------------------------------------
void cclk_lo(void)
{
	FPGA_CCLK_PORT->BSRR = (uint32_t)FPGA_CCLK_PIN << 16;
}
//--------------------------------------------------------------
void cclk_hi(void)
{
	FPGA_CCLK_PORT->BSRR = FPGA_CCLK_PIN;
}
//--------------------------------------------------------------
void data_lo(void)
{
	FPGA_DATA_PORT->BSRR = (uint32_t)FPGA_DATA_PIN << 16;
}
//--------------------------------------------------------------
void data_hi(void)
{
	FPGA_DATA_PORT->BSRR = FPGA_DATA_PIN;
}
//--------------------------------------------------------------
uint8_t read_initb(void)
{
	uint8_t value;

	value = HAL_GPIO_ReadPin(FPGA_INIT_B_PORT, FPGA_INIT_B_PIN);

	return value;
}
//--------------------------------------------------------------
uint8_t read_done(void)
{
	uint8_t value;

	value = HAL_GPIO_ReadPin(FPGA_DONE_PORT, FPGA_DONE_PIN);

	return value;
}
//--------------------------------------------------------------
void fpga_delay(uint32_t delay)
{
	uint32_t n;

	for(n=0;n<delay;n++) {
		// nothing to do
	}
}
